/* 0xWS2812 16-Channel WS2812 interface library
 * http://www.cnblogs.com/shangdawei/p/4762035.html
 * Copyright (c) 2014 Elia Ritterbusch, http://eliaselectronics.com
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */
#define NUM_LEDS 11	//ilosc diod w jednym ramieniu

#include <inttypes.h>
#include <stdio.h>
#include <stdint.h>
#include <stm32f10x.h>
#include "fire2012.h"

/* this define sets the number of TIM2 overflows to append to the data frame for the LEDs to load the received data into their registers */
#define WS2812_DEADPERIOD 39

/* okresla globalna jasnosc gwiazdy*/
#define GLOBAL_BRIGHTNESS 128

struct rgb
{
	uint8_t b;
	uint8_t g;
	uint8_t r;
	uint8_t x;
};

uint16_t WS2812_IO_High = 0xFFFF;
uint16_t WS2812_IO_Low = 0x0000;

volatile uint8_t WS2812_TC = 1;
volatile uint8_t TIM2_overflows = 0;

/* WS2812 framebuffer buffersize = (#LEDs / 16) * 24 */
#define WS2812_BUFFER_SIZE NUM_LEDS*24	//11 diod przypada na jeden kanal sterujacy
uint16_t WS2812_IO_framedata[ WS2812_BUFFER_SIZE ];

/* DMA1 Channel7 Interrupt Handler gets executed once the complete framebuffer has been transmitted to the LEDs */
void DMA1_Channel7_IRQHandler( void )
{
  // clear DMA7 transfer complete interrupt flag
  DMA_ClearITPendingBit( DMA1_IT_TC7 );
  // enable TIM2 Update interrupt to append 50us dead period
  TIM_ITConfig( TIM2, TIM_IT_Update, ENABLE );
  // disable the DMA channels
  DMA_Cmd( DMA1_Channel2, DISABLE );
  DMA_Cmd( DMA1_Channel5, DISABLE );
  DMA_Cmd( DMA1_Channel7, DISABLE );
  // IMPORTANT: disable the DMA requests, too!
  TIM_DMACmd( TIM2, TIM_DMA_CC1, DISABLE );
  TIM_DMACmd( TIM2, TIM_DMA_CC2, DISABLE );
  TIM_DMACmd( TIM2, TIM_DMA_Update, DISABLE );

}

/* TIM2 Interrupt Handler gets executed on every TIM2 Update if enabled */
void TIM2_IRQHandler( void )
{
  // Clear TIM2 Interrupt Flag
  TIM_ClearITPendingBit( TIM2, TIM_IT_Update );

  /* check if certain number of overflows has occured yet
   * this ISR is used to guarantee a 50us dead time on the data lines
   * before another frame is transmitted */
  if ( TIM2_overflows < (uint8_t) WS2812_DEADPERIOD )
  {
    // count the number of occured overflows
    TIM2_overflows++;
  }
  else
  {
    // clear the number of overflows
    TIM2_overflows = 0;
    // stop TIM2 now because dead period has been reached
    TIM_Cmd( TIM2, DISABLE );
    /* disable the TIM2 Update interrupt again
     * so it doesn't occur while transmitting data */
    TIM_ITConfig( TIM2, TIM_IT_Update, DISABLE );
    // finally indicate that the data frame has been transmitted
    WS2812_TC = 1;
  }
}

/* simple delay counter to waste time, don't rely on for accurate timing */
void Delay(uint32_t nCount)
{
  while(nCount--)
  {
  }
}

void GPIO_init( void )
{
  GPIO_InitTypeDef GPIO_InitStructure;
  // GPIOA Periph clock enable
  RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOA, ENABLE );
  // GPIOA pins WS2812 data outputs
  GPIO_InitStructure.GPIO_Pin = 0xfffF;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_Init( GPIOA, &GPIO_InitStructure );

  //red LED
  RCC_APB2PeriphClockCmd( RCC_APB2Periph_GPIOD, ENABLE );
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;
  GPIO_Init( GPIOD, &GPIO_InitStructure );

}

void TIM2_init( void )
{
  TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
  TIM_OCInitTypeDef TIM_OCInitStructure;
  NVIC_InitTypeDef NVIC_InitStructure;

  uint16_t PrescalerValue;

  // TIM2 Periph clock enable
  RCC_APB1PeriphClockCmd( RCC_APB1Periph_TIM2, ENABLE );

  PrescalerValue = (uint16_t) ( SystemCoreClock / 24000000 ) - 1;
  /* Time base configuration */
  TIM_TimeBaseStructure.TIM_Period = 29; // 800kHz
  TIM_TimeBaseStructure.TIM_Prescaler = PrescalerValue;
  TIM_TimeBaseStructure.TIM_ClockDivision = 0;
  TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
  TIM_TimeBaseInit( TIM2, &TIM_TimeBaseStructure );

  TIM_ARRPreloadConfig( TIM2, DISABLE );

  /* Timing Mode configuration: Channel 1 */
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_Timing;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Disable;
  TIM_OCInitStructure.TIM_Pulse = 8;
  TIM_OC1Init( TIM2, &TIM_OCInitStructure );
  TIM_OC1PreloadConfig( TIM2, TIM_OCPreload_Disable );

  /* Timing Mode configuration: Channel 2 */
  TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM1;
  TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Disable;
  TIM_OCInitStructure.TIM_Pulse = 17;
  TIM_OC2Init( TIM2, &TIM_OCInitStructure );
  TIM_OC2PreloadConfig( TIM2, TIM_OCPreload_Disable );

  /* configure TIM2 interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 2;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init( &NVIC_InitStructure );
}

void DMA_init( void )
{
  DMA_InitTypeDef DMA_InitStructure;
  NVIC_InitTypeDef NVIC_InitStructure;

  RCC_AHBPeriphClockCmd( RCC_AHBPeriph_DMA1, ENABLE );

  // TIM2 Update event
  /* DMA1 Channel2 configuration ----------------------------------------------*/
  DMA_DeInit( DMA1_Channel2 );
  DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t) &GPIOA->ODR;
  DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t) &WS2812_IO_High;
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
  DMA_InitStructure.DMA_BufferSize = 0;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Disable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
  DMA_Init( DMA1_Channel2, &DMA_InitStructure );

  // TIM2 CC1 event
  /* DMA1 Channel5 configuration ----------------------------------------------*/
  DMA_DeInit( DMA1_Channel5 );
  DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t) &GPIOA->ODR;
  DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t) &WS2812_IO_framedata;
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
  DMA_InitStructure.DMA_BufferSize = 0;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
  DMA_Init( DMA1_Channel5, &DMA_InitStructure );

  // TIM2 CC2 event
  /* DMA1 Channel7 configuration ----------------------------------------------*/
  DMA_DeInit( DMA1_Channel7 );
  DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t) &GPIOA->ODR;
  DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t) &WS2812_IO_Low;
  DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
  DMA_InitStructure.DMA_BufferSize = 0;
  DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
  DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Disable;
  DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
  DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
  DMA_InitStructure.DMA_Mode = DMA_Mode_Normal;
  DMA_InitStructure.DMA_Priority = DMA_Priority_High;
  DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
  DMA_Init( DMA1_Channel7, &DMA_InitStructure );

  /* configure DMA1 Channel7 interrupt */
  NVIC_InitStructure.NVIC_IRQChannel = DMA1_Channel7_IRQn;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init( &NVIC_InitStructure );
  /* enable DMA1 Channel7 transfer complete interrupt */
  DMA_ITConfig( DMA1_Channel7, DMA_IT_TC, ENABLE );
}

/* Transmit the frambuffer with buffersize number of bytes to the LEDs
 * buffersize = (#LEDs / 16) * 24 */
void WS2812_sendbuf( uint32_t buffersize )
{
  // transmission complete flag, indicate that transmission is taking place
  WS2812_TC = 0;

  // clear all relevant DMA flags
  DMA_ClearFlag( DMA1_FLAG_TC2 | DMA1_FLAG_HT2 | DMA1_FLAG_GL2 | DMA1_FLAG_TE2 );
  DMA_ClearFlag( DMA1_FLAG_TC5 | DMA1_FLAG_HT5 | DMA1_FLAG_GL5 | DMA1_FLAG_TE5 );
  DMA_ClearFlag( DMA1_FLAG_HT7 | DMA1_FLAG_GL7 | DMA1_FLAG_TE7 );

  // configure the number of bytes to be transferred by the DMA controller
  DMA_SetCurrDataCounter( DMA1_Channel2, buffersize );
  DMA_SetCurrDataCounter( DMA1_Channel5, buffersize );
  DMA_SetCurrDataCounter( DMA1_Channel7, buffersize );

  // clear all TIM2 flags
  TIM2->SR = 0;

  // enable the corresponding DMA channels
  DMA_Cmd( DMA1_Channel2, ENABLE );
  DMA_Cmd( DMA1_Channel5, ENABLE );
  DMA_Cmd( DMA1_Channel7, ENABLE );

  // IMPORTANT: enable the TIM2 DMA requests AFTER enabling the DMA channels!
  TIM_DMACmd( TIM2, TIM_DMA_CC1, ENABLE );
  TIM_DMACmd( TIM2, TIM_DMA_CC2, ENABLE );
  TIM_DMACmd( TIM2, TIM_DMA_Update, ENABLE );

  // preload counter with 29 so TIM2 generates UEV directly to start DMA transfer
  TIM_SetCounter( TIM2, 29 );

  // start TIM2
  TIM_Cmd( TIM2, ENABLE );
}



/* This function sets the color of a single pixel in the framebuffer
 *
 * Arguments:
 * row = the channel number/LED strip the pixel is in from 0 to 15
 * column = the column/LED position in the LED string from 0 to number of LEDs per strip
 * red, green, blue = the RGB color triplet that the pixel should display
 */
void WS2812_framedata_setPixel( uint8_t row, uint16_t column, uint8_t red, uint8_t green, uint8_t blue )
{
  uint8_t i;
  for ( i = 0; i < 8; i++ )
  {
    // clear the data for pixel
    WS2812_IO_framedata[ ( ( column * 24 ) + i ) ] &= ~( 0x01 << row );
    WS2812_IO_framedata[ ( ( column * 24 ) + 8 + i ) ] &= ~( 0x01 << row );
    WS2812_IO_framedata[ ( ( column * 24 ) + 16 + i ) ] &= ~( 0x01 << row );
    // write new data for pixel
    WS2812_IO_framedata[ ( ( column * 24 ) + i ) ] |= ( ( ( ( green << i ) & 0x80 ) >> 7 ) << row );
    WS2812_IO_framedata[ ( ( column * 24 ) + 8 + i ) ] |= ( ( ( ( red << i ) & 0x80 ) >> 7 ) << row );
    WS2812_IO_framedata[ ( ( column * 24 ) + 16 + i ) ] |= ( ( ( ( blue << i ) & 0x80 ) >> 7 ) << row );
  }
}

//wraper na funkcje _setPixel, z tym ze dana wejrsicowa jest wartosc RGB
void WS2812_framedata_setPixelRGB(uint8_t row, uint16_t column, uint32_t rgb)
{
	uint8_t red, green, blue;
	union {
		uint32_t rgb;
		struct{
			uint8_t x;
			uint8_t r;
			uint8_t g;
			uint8_t b;
		}b;
	}unRGB;

	unRGB.rgb = rgb;
	WS2812_framedata_setPixel(row, column, unRGB.b.r, unRGB.b.g, unRGB.b.b);
}


/* This function is a wrapper function to set all LEDs in the complete row to the specified color
 *
 * Arguments:
 * row = the channel number/LED strip to set the color of from 0 to 15
 * columns = the number of LEDs in the strip to set to the color from 0 to number of LEDs per strip
 * red, green, blue = the RGB color triplet that the pixels should display

void WS2812_framedata_setRow( uint8_t row, uint16_t columns, uint8_t red,
  uint8_t green, uint8_t blue )
{
  uint8_t i;
  for ( i = 0; i < columns; i++ )
  {
    WS2812_framedata_setPixel( row, i, red, green, blue );
  }
}
 */
/* This function is a wrapper function to set all the LEDs in the column to the specified color
 *
 * Arguments:
 * rows = the number of channels/LED strips to set the row in from 0 to 15
 * column = the column/LED position in the LED string from 0 to number of LEDs per strip
 * red, green, blue = the RGB color triplet that the pixels should display

void WS2812_framedata_setColumn( uint8_t rows, uint16_t column, uint8_t red,
  uint8_t green, uint8_t blue )
{
  uint8_t i;
  for ( i = 0; i < rows; i++ )
  {
    WS2812_framedata_setPixel( i, column, red, green, blue );
  }
}
 */
struct rgb HSV_to_RGB(uint16_t Hue, uint8_t Sat, uint8_t Val)
{
	uint8_t h2;
	uint16_t Chroma, h1, x, h, m;

	union kolor
	{
		struct rgb col;
		uint32_t rgb;
	}result;

	//bezpiecznik
	Hue %= 360;
	result.rgb = 0;

	//obliczenie Chroma
	Chroma = ((uint16_t)Val*Sat) >> 8;

	h1 = (Hue<<6) / 15;
	h2 = (uint8_t)(Hue / 60);
	m = (uint16_t)Val - Chroma;

	h = h1 % 512;
	if (h > 255)
		h = 512 - h;

	x = (Chroma * h) >> 8;

	if (h2 == 0)
	{
		result.col.r = Chroma + m;
		result.col.g = x + m;
		result.col.b = m;
	}
	else if (h2 == 1)
	{
		result.col.r = x + m;
		result.col.g = Chroma + m;
		result.col.b = m;
	}
	else if (h2 == 2)
	{
		result.col.r = m;
		result.col.g = Chroma + m;
		result.col.b = x + m;
	}
	else if (h2 == 3)
	{
		result.col.r = m;
		result.col.g = x + m;
		result.col.b = Chroma + m;
	}
	else if (h2 == 4)
	{
		result.col.r = x + m;
		result.col.g = m;
		result.col.b = Chroma + m;
	}
	else if (h2 == 5)
	{
		result.col.r = Chroma + m;
		result.col.g = m;
		result.col.b = x + m;
	}
	else
		result.rgb = 0;

	return result.col;
}

void WS2812_framedata_setPixel_HSV( uint8_t row, uint16_t column, uint16_t hue, uint8_t sat, uint8_t val )
{
  struct rgb ColRgb;

  ColRgb = HSV_to_RGB(hue, sat, val);
  WS2812_framedata_setPixel(row, column, ColRgb.r, ColRgb.g, ColRgb.b);

}

//arm: 1...6; arm=0 to dioda centralna
void WS2812_star_setLongArm_HSV (uint8_t arm, uint16_t hue, uint8_t sat, uint8_t val)
{
	uint8_t i;

    for ( i = 0; i < 8; i++ )
	{
		WS2812_framedata_setPixel_HSV( arm % 7, i, hue, sat, val );
	}
}

//arm: 1...6; arm=0 to dioda centralna
void WS2812_star_setShortArm_HSV (uint8_t arm, uint16_t hue, uint8_t sat, uint8_t val)
{
	uint8_t i;

    for ( i = 8; i < 11; i++ )
	{
		WS2812_framedata_setPixel_HSV( arm % 7, i, hue, sat, val);
	}
}

//zapala ramie o zadanym indeksie (obojetnie czy dlugie czy krotkie, ramie 0 to dioda centralna)
void WS2812_star_setArm_HSV (uint8_t arm, uint16_t hue, uint8_t sat, uint8_t val)
{
	uint8_t reindex[] = {0, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 1};

	arm %= 13;	//zabezpieczenie na wypadek nieprawidlowej wartosci arm

	if (arm == 0)
	{
		WS2812_framedata_setPixel_HSV( 0, 0, hue, sat, val);
	}
	else if (arm % 2)
	{
		WS2812_star_setLongArm_HSV(reindex[arm], hue, sat, val);
	}
	else
	{
		WS2812_star_setShortArm_HSV(reindex[arm], hue, sat, val);
	}
}

//ustawia wskazowke godzinowa zegara (trzy diody idx=2,3,4 dlugiego ramienia lub 3 diody krotkiego ramienia) - imitacja krotkiej wskazowki zegara
void WS2812_star_setClockHArrow_HSV (uint8_t arm, uint16_t hue, uint8_t sat, uint8_t val)
{
	uint8_t reindex[] = {0, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 1};

	arm %= 13;	//zabezpieczenie na wypadek nieprawidlowej wartosci arm

	if (arm == 0)
	{
		WS2812_framedata_setPixel_HSV( 0, 0, hue, sat, val);
	}
	else if (arm % 2)
	{
		WS2812_star_setLongArm_HSV(reindex[arm], hue, sat, val);
		//wygas pozostale diody
		WS2812_framedata_setPixel(reindex[arm], 0, 0,0,0);
		WS2812_framedata_setPixel(reindex[arm], 1, 0,0,0);
		WS2812_framedata_setPixel(reindex[arm], 5, 0,0,0);
		WS2812_framedata_setPixel(reindex[arm], 6, 0,0,0);
		WS2812_framedata_setPixel(reindex[arm], 7, 0,0,0);
	}
	else
	{
		WS2812_star_setShortArm_HSV(reindex[arm], hue, sat, val);
	}
}

void WS2812_star_setLongRadius_HSV (uint8_t radius, uint16_t hue, uint8_t sat, uint8_t val)
{
	uint8_t j;

	if (radius == 0)	//zapal centralna diode
	{
		WS2812_framedata_setPixel_HSV( 0, 0, hue, sat, val );
	}
	else
	{
		for (j = 1; j<7; j++) //petla po ramionach
		{
			WS2812_framedata_setPixel_HSV( j, radius-1, hue, sat, val );
		}
	}
}

void WS2812_star_setShortRadius_HSV (uint8_t radius, uint16_t hue, uint8_t sat, uint8_t val)
{

	if (radius == 0)
		WS2812_star_setLongRadius_HSV(0, hue, sat, val);
	else
		WS2812_star_setLongRadius_HSV(12-radius, hue, sat, val);
}

#define M 2147483647 /* 2^31 - 1 (A large prime number) */
#define A 16807      /* Prime root of M, passes statistical tests and produces a full cycle */
#define Q 127773 /* M / A (To avoid overflow on A * seed) */
#define R 2836   /* M % A (To avoid overflow on A * seed) */

uint32_t lcg_parkmiller(uint32_t seed)
{
    uint32_t hi = seed / Q;
    uint32_t lo = seed % Q;
    int32_t test = A * lo - R * hi;
    if (test <= 0)
        test += M;
    return test;
}

// HeatColor( uint8_t temperature)
//
// Approximates a 'black body radiation' spectrum for
// a given 'heat' level.  This is useful for animations of 'fire'.
// Heat is specified as an arbitrary scale from 0 (cool) to 255 (hot).
// This is NOT a chromatically correct 'black body radiation'
// spectrum, but it's surprisingly close, and it's fast and small.
//
// On AVR/Arduino, this typically takes around 70 bytes of program memory,
// versus 768 bytes for a full 256-entry RGB lookup table.

struct rgb HeatColor( uint8_t temperature)
{
   struct rgb heatcolor;

	// Scale 'heat' down from 0-255 to 0-191,
	// which can then be easily divided into three
	// equal 'thirds' of 64 units each.
    // TB jest to kopia funkcji scale8_video() z FastLED
    uint8_t t192 = (((int)temperature * (int)192) >> 8) + ((temperature && 192)?1:0);

	// calculate a value that ramps up from
	// zero to 255 in each 'third' of the scale.
	uint8_t heatramp = t192 & 0x3F; // 0..63
	heatramp <<= 2; // scale up to 0..252

	// now figure out which third of the spectrum we're in:
	if( t192 & 0x80) {
		// we're in the hottest third
		heatcolor.r = 255; // full red
		heatcolor.g = 255; // full green
		heatcolor.b = heatramp; // ramp up blue

	} else if( t192 & 0x40 ) {
		// we're in the middle third
		heatcolor.r = 255; // full red
		heatcolor.g = heatramp; // ramp up green
		heatcolor.b = 0; // no blue

	} else {
		// we're in the coolest third
		heatcolor.r = heatramp; // ramp up red
		heatcolor.g = 0; // no green
		heatcolor.b = 0; // no blue
	}

	return heatcolor;
}


// Fire2012 by Mark Kriegsman, July 2012
// as part of "Five Elements" shown here: http://youtu.be/knWiGsmgycY
////
// This basic one-dimensional 'fire' simulation works roughly as follows:
// There's a underlying array of 'heat' cells, that model the temperature
// at each point along the line.  Every cycle through the simulation,
// four steps are performed:
//  1) All cells cool down a little bit, losing heat to the air
//  2) The heat from each cell drifts 'up' and diffuses a little
//  3) Sometimes randomly new 'sparks' of heat are added at the bottom
//  4) The heat from each cell is rendered as a color into the leds array
//     The heat-to-color mapping uses a black-body radiation approximation.
//
// Temperature is in arbitrary units from 0 (cold black) to 255 (white hot).
//
// This simulation scales it self a bit depending on NUM_LEDS; it should look
// "OK" on anywhere from 20 to 100 LEDs without too much tweaking.
//
// I recommend running this simulation at anywhere from 30-100 frames per second,
// meaning an interframe delay of about 10-35 milliseconds.
//
// Looks best on a high-density LED setup (60+ pixels/meter).
//
//
// There are two main parameters you can play with to control the look and
// feel of your fire: COOLING (used in step 1 above), and SPARKING (used
// in step 3 above).
//
// COOLING: How much does the air cool as it rises?
// Less cooling = taller flames.  More cooling = shorter flames.
// Default 50, suggested range 20-100
#define COOLING  50

// SPARKING: What chance (out of 255) is there that a new spark will be lit?
// Higher chance = more roaring fire.  Lower chance = more flickery fire.
// Default 120, suggested range 50-200.
#define SPARKING 50

void Fire2012(uint8_t gReverseDirection, uint8_t arm)
{
// Array of temperature readings at each simulation cell
  static uint8_t heat[NUM_LEDS];
  uint32_t i, j, k;

  // Step 1.  Cool down every cell a little
    for( i = 0; i < NUM_LEDS; i++) {
      heat[i] = qsub8( heat[i],  random8ML(0, ((COOLING * 10) / NUM_LEDS) + 2));
    }

    // Step 2.  Heat from each cell drifts 'up' and diffuses a little
    for( k= NUM_LEDS - 1; k >= 2; k--) {
      heat[k] = (heat[k - 1] + heat[k - 2] + heat[k - 2] ) / 3;
    }

    // Step 3.  Randomly ignite new 'sparks' of heat near the bottom
    if( random8() < SPARKING ) {
      int y = random8L(7);
      heat[y] = qadd8( heat[y], random8ML(160,255) );
    }

    // Step 4.  Map from heat cells to LED colors
    for( j = 0; j < NUM_LEDS; j++) {
      struct rgb color = HeatColor( heat[j]);
      int pixelnumber;
      if( gReverseDirection ) {
        pixelnumber = (NUM_LEDS-1) - j;
      } else {
        pixelnumber = j;
      }

      if (arm == 0)
    	  pixelnumber = 0;	//arm=0 oznacza diode centralna
      WS2812_framedata_setPixel(arm,pixelnumber, color.r, color.g, color.b);
      //leds[pixelnumber] = color;
      if (arm == 0) break;	//arm=0 oznacza diode centralna
    }
}

//fala radialna z wygaszeniem przez bialy blysk
void gwiazdka_schemat_1(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t radius, fade;

	//ilosc powtorzen danego programu
	while(cykle--)
	{
		for (radius = 0; radius < 12; radius++)
		{
			// wait until the last frame was transmitted
			while ( !WS2812_TC );

			WS2812_star_setLongRadius_HSV(radius, radius*45, 255, GLOBAL_BRIGHTNESS);
			// send the framebuffer out to the LEDs
			WS2812_sendbuf( WS2812_BUFFER_SIZE );
			// wait some amount of time
			Delay( (uint32_t)(1<<24) >> szybkosc );
		}

		fade = 255;
		do
		{
			fade--;
			for (radius = 0; radius < 12; radius++)
			{
				// wait until the last frame was transmitted
				while ( !WS2812_TC );

				WS2812_star_setLongRadius_HSV(radius, radius*45, 255-fade, fade);
				// send the framebuffer out to the LEDs
				WS2812_sendbuf( WS2812_BUFFER_SIZE );
				// wait some amount of time
				//Delay( (uint32_t)(1<<6) >> szybkosc);
			}
		}while(fade);
	}
}

//fala radialna - wypelnienie kolorem
void gwiazdka_schemat_2(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t radius, hue, sat, val;

	//ilosc powtorzen danego programu
	while(cykle--)
	{
		for (hue=0; hue<6; hue++)
		{
			for (radius = 0; radius < 12; radius++)
			{
				// wait until the last frame was transmitted
				while ( !WS2812_TC );

				WS2812_star_setLongRadius_HSV(radius, hue*60, 255, GLOBAL_BRIGHTNESS);
				// send the framebuffer out to the LEDs
				WS2812_sendbuf( WS2812_BUFFER_SIZE );
				// wait some amount of time
				Delay( (uint32_t)(1<<24) >> szybkosc );
			}
		}
	}
}

//fala radialna tecza z wygaszaniem poprzedniego kregu
void gwiazdka_schemat_3(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t radius, hue, sat, val, r;

	//ilosc powtorzen danego programu
	while(cykle--)
	{
		for (radius = 0; radius < 12; radius++)
		{
			// wait until the last frame was transmitted
			while ( !WS2812_TC );

			r = radius * 60;
			hue = 255;
			if (radius > 360)
			{
				r = 0;
				hue = 0;
			}

			WS2812_star_setLongRadius_HSV(radius, r, hue, GLOBAL_BRIGHTNESS);
			if (radius == 0)
			{
				WS2812_star_setLongRadius_HSV(11, 0, 0, 0);	//zgas ostatnie diody w ramionach
			}
			else
			{
				WS2812_star_setLongRadius_HSV(radius-1, 0, 0, 0);	//zgas poprzednie diody w ramionach
			}
			// send the framebuffer out to the LEDs
			WS2812_sendbuf( WS2812_BUFFER_SIZE );
			// wait some amount of time
			Delay( (uint32_t)(1<<24) >> szybkosc );
		}

	}
}

//pulsowanie gwiazdki w roznych kolorach - na wzor bicia serca
void gwiazdka_schemat_4(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t radius, hue, sat, val, fade, i;


	hue = 0;

	//ilosc powtorzen danego programu
	while(cykle--)
	{

		for (hue = 0; hue<360; hue+=60)
		{
			for (i = 0; i<2; i++)
			{
				// wait until the last frame was transmitted
				while ( !WS2812_TC );
				//zapal wszystkie diody na raz
				for (radius = 0; radius < 9; radius++)
				{
					WS2812_star_setLongRadius_HSV(radius, hue, 255, GLOBAL_BRIGHTNESS);
					// send the framebuffer out to the LEDs
				}
				for (radius = 1; radius < 4; radius++)
				{
					WS2812_star_setShortRadius_HSV(radius, hue, 255, GLOBAL_BRIGHTNESS);
					// send the framebuffer out to the LEDs
				}
				WS2812_sendbuf( WS2812_BUFFER_SIZE );

				Delay( (uint32_t)(1<<18) );

				fade = GLOBAL_BRIGHTNESS;
				do
				{
					if (fade > 16 )
						fade-=(7-3*i);

					for (radius = 0; radius < 12; radius++)
					{
						// wait until the last frame was transmitted
						while ( !WS2812_TC );

						WS2812_star_setLongRadius_HSV(radius, hue, 255, fade);
						// send the framebuffer out to the LEDs
							// send the framebuffer out to the LEDs
						WS2812_sendbuf( WS2812_BUFFER_SIZE );
						// wait some amount of time
						//Delay( (uint32_t)(1<<8) >> szybkosc);
					}
				}while(fade > 16);
			}
			Delay( (uint32_t)(1<<26) >> szybkosc);
		}
	}
}

//losowe mruganie przypadkowych diodek z pozostawieniem przytlumionego tla
void gwiazdka_schemat_5(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t radius, hue, sat, val, rnd, idx, ic, ir, i;
	static uint32_t seed = 12345;
	ir = ic = hue = 0;

	//ilosc powtorzen danego programu
	while(cykle--)
	{
		for (i=0; i<2000; i++)
		{
			while ( !WS2812_TC );

			WS2812_framedata_setPixel_HSV(ir, ic, hue*120, 255, 8);	//wygas poprzednia diode

			seed = lcg_parkmiller(seed);

			if (i < 1000)
				hue = seed % 3;

			idx = seed % 67;
			if (idx == 66)
			{
				ir = 0;
				ic = 0;
			}
			else
			{
				ir = (idx / 11) + 1;
				ic = idx % 11;
			}

			WS2812_framedata_setPixel(ir, ic, 255, 255, 255);	//zapal kolejna diode

			// send the framebuffer out to the LEDs
			WS2812_sendbuf( WS2812_BUFFER_SIZE );

			Delay( (uint32_t)(1<<18) >> szybkosc);

		}

		//wygas ostatnia diode i odczekaj chwile
		while ( !WS2812_TC );
		WS2812_framedata_setPixel_HSV(ir, ic, hue*120, 255, 8);	//wygas poprzednia diode
		WS2812_sendbuf( WS2812_BUFFER_SIZE );
		Delay( (uint32_t)(1<<24) >> szybkosc);
	}
}

//swieci caa gwiazdka zmieniajac plynnie kolory, konczy na bialym
void gwiazdka_schemat_6(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t radius, hue, sat, val;


	//ilosc powtorzen danego programu
	while(cykle--)
	{
		sat = 255;
		for (hue = 0; hue<400; hue++)
		{

			if (hue > 360)
			{
				if (sat > 10)
				{
					sat -= 10;
				}
				else
				{
					sat = 0;
				}
			}

			// wait until the last frame was transmitted
			while ( !WS2812_TC );
			//zapal wszystkie diody na raz
			for (radius = 0; radius < 9; radius++)
			{
				WS2812_star_setLongRadius_HSV(radius, hue, sat, GLOBAL_BRIGHTNESS);
			}
			for (radius = 1; radius < 4; radius++)
			{
				WS2812_star_setShortRadius_HSV(radius, hue, sat, GLOBAL_BRIGHTNESS);
			}
			WS2812_sendbuf( WS2812_BUFFER_SIZE );

			Delay( (uint32_t)(1<<18) );
		}
	}
}

//efekt ognia
void gwiazdka_schemat_7(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t t, hue, sat, val, i;
	static seed = 12345;

	seed = lcg_parkmiller(seed);

	//ilosc powtorzen danego programu
	while(cykle--)
	{
		for (t=0; t<1000; t++)
		{
			random16_add_entropy( seed % 65536 );

			// wait until the last frame was transmitted
			while ( !WS2812_TC );

			//wygeneruj efekt ognia dla poszczegolnych ramion
			for (i=0; i<7; i++)
				Fire2012(0, i);

			WS2812_sendbuf( WS2812_BUFFER_SIZE );
			Delay( (uint32_t)0x16C16 );
		}
	}
}

//efekt radaru
void gwiazdka_schemat_8(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t i, a, idx, arm, hue, sat, val, dhue;
	uint8_t vals[12] ={255,100,70,49,34,24,17,12,8,6,4,2};
	uint16_t hues[12]={0};

	hue = 0;
	sat = 255;
	val = GLOBAL_BRIGHTNESS;

	WS2812_star_setArm_HSV(0, hue, 0, val/2);	//centralna dioda biala

	//ilosc powtorzen danego programu
	while(cykle--)
	{
		//petla cyklu ruchu ramienia
		for (i=1; i<72; i++)
		{
			idx = i % 12 + 1;
			if (idx == 1)
			{
				hue += 15;
				if (hue >= 360)
					hue = 0;
			}
			hues[idx-1] = hue;

			while (!WS2812_TC);

			//zapis wszystkich ramion z obnizonymi jasnosciami
			for (a = 1; a < 13; a++)
			{
				WS2812_star_setArm_HSV(idx, hues[idx-1], sat, vals[a-1]);
				idx--;
				if (!idx)
					idx = 12;
			}

			WS2812_sendbuf( WS2812_BUFFER_SIZE );

			Delay( (uint32_t) (1<< 24) >> szybkosc );

		}

	}
}

void gwiazdka_schemat_9(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t i, hue, sat, val;

	//ilosc powtorzen danego programu
	while(cykle--)
	{
		for (i=0; i<13; i++)
		{
			while (!WS2812_TC);

			WS2812_star_setArm_HSV(i, 0, 255, GLOBAL_BRIGHTNESS);
			//zmien wzorzec kolorow dla kazdej diody

			WS2812_sendbuf( WS2812_BUFFER_SIZE );
			Delay( (uint32_t) (1<< 24) >> szybkosc );
		}
	}
}

void gwiazdka_schemat_10(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t i, k, hue, sat, val;
	uint8_t reindex[] = {0, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 1};
	static seed = 54321;

	sat = 255;
	val = GLOBAL_BRIGHTNESS;

	//ilosc powtorzen danego programu
	while(cykle--)
	{
		seed = lcg_parkmiller(seed);
		hue = seed % 360;

		gwiazdka_erase();
		for (k=1; k<=5; k+=2)
		{
			//dlugie ramie 1
			i=8;
			while(i)
			{
				i--;
				while (!WS2812_TC);
				WS2812_framedata_setPixel_HSV( reindex[k], i, hue, sat, val );
				WS2812_sendbuf( WS2812_BUFFER_SIZE );
				Delay( (uint32_t) (1<< 24) >> szybkosc );
			}
			//centralna dioda
			while (!WS2812_TC);
			WS2812_framedata_setPixel_HSV( 0, 0, hue, sat, val );
			WS2812_sendbuf( WS2812_BUFFER_SIZE );
			Delay( (uint32_t) (1<< 24) >> szybkosc );

			//ramie przeciwlegle
			for (i=0; i<8; i++)
			{
				while (!WS2812_TC);
				WS2812_framedata_setPixel_HSV( reindex[k+6], i, hue, sat, val );
				WS2812_sendbuf( WS2812_BUFFER_SIZE );
				Delay( (uint32_t) (1<< 24) >> szybkosc );
			}

		}
		//krotkie ramiona
		for (k=2; k<=6; k+=2)
		{
			//ramie krotkie
			for (i=8; i<11; i++)
			{
				while (!WS2812_TC);
				WS2812_framedata_setPixel_HSV( reindex[k+6], i, hue, sat, val );
				WS2812_sendbuf( WS2812_BUFFER_SIZE );
				Delay( (uint32_t) (1<< 24) >> szybkosc );
			}

			//centralna dioda
			while (!WS2812_TC);
			WS2812_framedata_setPixel_HSV( 0, 0, hue, sat, val );
			WS2812_sendbuf( WS2812_BUFFER_SIZE );
			Delay( (uint32_t) (1<< 24) >> szybkosc );

			//ramie krotkie przeciwlegle
			i=11;
			while(i >= 8)
			{
				i--;
				while (!WS2812_TC);
				WS2812_framedata_setPixel_HSV( reindex[k], i, hue, sat, val );
				WS2812_sendbuf( WS2812_BUFFER_SIZE );
				Delay( (uint32_t) (1<< 24) >> szybkosc );
			}
		}
	}
}

/*
void gwiazdka_schemat_11(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t i, hue, sat, val;
	uint16_t Tmet[] = {1300,1224,1155,1093,1036,985,939,897,859,825,794,766,741,718,697,679,662,646,632,620,608,598,589,580,573,566,559,554,549,544,540};
	uint32_t SpectrumMet[] = {0xFFFFFF, 0xFFFBDF, 0xFFF9D5, 0xFFF7C4, 0xFFF09D, 0xFFDC91, 0xFFD382, 0xFEAD5C, 0xFE823C, 0xFE6D30, 0xFE5710, 0xFD5010, 0xFD460F, 0xF03E0A, 0xEA3605, 0xD21903, 0xBF0A02, 0x8C0501, 0x6B0101, 0x5A0101, 0x500101, 0x480000, 0x400000, 0x360000, 0x300000, 0x2A0000, 0x240000, 0x1F0000, 0x1B0000, 0x160000, 0x140000};
	uint16_t Tdiody[11] = {0};

	//ilosc powtorzen danego programu
	while(cykle--)
	{

		//wylacz wszystkie diody
		while (!WS2812_TC);
		gwiazdka_erase();
		//zmien wzorzec kolorow dla kazdej diody
		WS2812_sendbuf( WS2812_BUFFER_SIZE );

		for (i=0; i<11; i++)
		{
			while (!WS2812_TC);


			Delay( (uint32_t) (1<< 24) >> szybkosc );
		}
	}
}
*/

void gwiazdka_schemat_12(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t i, k, hue, hue2, sat, val;

	//ilosc powtorzen danego programu

	val = GLOBAL_BRIGHTNESS;
	sat = 255;

	while(cykle--)
	{
		for (k=0; k<360; k++)
		{
			while (!WS2812_TC);

			//zmien wzorzec kolorow dla kazdej diody

			WS2812_star_setLongRadius_HSV(0, k, sat, val);
			WS2812_star_setLongRadius_HSV(1, k, sat, val);
			WS2812_star_setShortRadius_HSV(3, k, sat, val);
			WS2812_star_setLongRadius_HSV(8, k, sat, val);

			//inny kolor
			for (i=2; i<8; i++)
			{
				WS2812_star_setLongRadius_HSV(i, (k+60)%360, sat, val/4);
			}
			WS2812_star_setShortRadius_HSV(1, (k+60)%360, sat, val/4);
			WS2812_star_setShortRadius_HSV(2, (k+60)%360, sat, val/4);

			WS2812_sendbuf( WS2812_BUFFER_SIZE );
			Delay( (uint32_t)(1<<24) >> szybkosc );
		}
	}
}

//kazde ramie w innym kolorze - obracajace sie kolory
void gwiazdka_schemat_13(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t i, a, hue, sat, val;

	val = GLOBAL_BRIGHTNESS;
	sat = 255;

	//ilosc powtorzen danego programu
	while(cykle--)
	{

		for(hue=0; hue<360; hue++)
		{
			while (!WS2812_TC);

			//centralna dioda zawsze biala
			//ale zmieniaj natezenie do polowy max. jasnosci
			WS2812_star_setArm_HSV(0, 0, 0, val - (hue % (GLOBAL_BRIGHTNESS/2)));

			//zapal kade rami w innym kolorze (z inkrementacja +30 do hue)
			//zmieniaj kolor kazdego ramiona cyklicznie
			for (a=1; a<13; a++)
			{
				WS2812_star_setArm_HSV(a, (hue + (a-1)*(360/12)) % 360, sat, val);
			}

			WS2812_sendbuf( WS2812_BUFFER_SIZE );
			Delay( (uint32_t) (1<< 24) >> szybkosc );
		}
	}
}

//imitacja zegara z duza i mala wskazowka w innych kolorach wybieranych przypadkowo
void gwiazdka_schemat_14(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t i, sat, val, m, h;
	static uint32_t mhue, hhue;
	static seed = 54321;

	seed = lcg_parkmiller(seed);
	mhue = seed % 360;
	hhue = (seed+180) % 360;

	sat = 255;
	val = GLOBAL_BRIGHTNESS;

	//ilosc powtorzen danego programu
	while(cykle--)
	{
		for (i=1; i<(12*6); i++)
		{
			while (!WS2812_TC);
			gwiazdka_erase();	//wyczysc gwiazdke
			WS2812_sendbuf( WS2812_BUFFER_SIZE );

			while (!WS2812_TC);
			WS2812_framedata_setPixel(0,0, val/4, val/4, val/4);	//centralna dioda biala
			h = i/6 + 1;
			m = i%6 + 1;
			WS2812_star_setClockHArrow_HSV(h, hhue,sat,val);
			WS2812_star_setLongArm_HSV(m, mhue, sat, val);

			WS2812_sendbuf( WS2812_BUFFER_SIZE );
			Delay( (uint32_t) (1<< 24) >> szybkosc );

		}
	}
}

void gwiazdka_schemat_X(uint32_t cykle, uint32_t szybkosc)
{
	uint32_t i, hue, sat, val;

	//ilosc powtorzen danego programu
	while(cykle--)
	{

		while (!WS2812_TC);

		//zmien wzorzec kolorow dla kazdej diody

		WS2812_sendbuf( WS2812_BUFFER_SIZE );
		Delay( (uint32_t) (1<< 24) >> szybkosc );

	}
}

void gwiazdka_erase(void)
{
	uint32_t i, hue, sat, val;

	//ilosc powtorzen danego programu
	while (!WS2812_TC);

	for (i=0; i<13; i++)
		WS2812_star_setArm_HSV(i, 0, 0, 0);

	WS2812_sendbuf( WS2812_BUFFER_SIZE );
}

int main( void )
{
  uint32_t radius, hue, sat, val;


  GPIO_SetBits(GPIOD, GPIO_Pin_2);
  Delay( (uint32_t)(1<<22));

  GPIO_init( );
  DMA_init( );
  TIM2_init( );

  GPIO_ResetBits(GPIOD, GPIO_Pin_2);
  GPIO_ResetBits(GPIOA, 0xffff);
  Delay( (uint32_t)(1<<20));


  GPIO_SetBits(GPIOD, GPIO_Pin_2);
  Delay( (uint32_t)(1<<20));


  while ( 1 )
  {

	  gwiazdka_schemat_14(1, 4);
	  gwiazdka_schemat_13(3, 7);
	  gwiazdka_schemat_12(3, 6);
	  gwiazdka_schemat_10(5, 5);
	  gwiazdka_schemat_8(8, 5);
	  gwiazdka_schemat_4(1, 2);
	  gwiazdka_schemat_4(1, 4);
	  gwiazdka_schemat_4(1, 6);

	  gwiazdka_schemat_7(1, 0);
	  gwiazdka_schemat_5(1, 2);
	  gwiazdka_schemat_3(10, 5);
	  gwiazdka_schemat_2(1, 5);

	  gwiazdka_schemat_6(1, 1);
	  gwiazdka_schemat_5(1, 2);

	  gwiazdka_schemat_1(5, 4);
	  gwiazdka_schemat_2(3, 5);
	  gwiazdka_schemat_3(10, 5);

  }
}
